/* File: variable.c */

/*
 * Copyright (c) 1997 Ben Harrison, James E. Wilson, Robert A. Koeneke
 *
 * This software may be copied and distributed for educational, research,
 * and not for profit purposes provided that this copyright and statement
 * are included in all such copies.  Other copyrights may also apply.
 */

#include "angband.h"

/*
 * Hack -- Link a copyright message into the executable
 */
cptr copyright =
	"Copyright (c) 1997 Ben Harrison, James E. Wilson, Robert A. Keoneke\n"
	"\n"
	"This software may be copied and distributed for educational, research,\n"
	"and not for profit purposes provided that this copyright and statement\n"
	"are included in all such copies.  Other copyrights may also apply.\n";

/*
 * Executable version
 */
byte version_major = VERSION_MAJOR;
byte version_minor = VERSION_MINOR;
byte version_patch = VERSION_PATCH;
byte version_extra = VERSION_EXTRA;

/*
 * Savefile version
 */
byte sf_major;			/* Savefile's "version_major" */
byte sf_minor;			/* Savefile's "version_minor" */
byte sf_patch;			/* Savefile's "version_patch" */
byte sf_extra;			/* Savefile's "version_extra" */

/*
 * Savefile information
 */
u32b sf_xtra;			/* Operating system info */
u32b sf_when;			/* Time when savefile created */
u16b sf_lives;			/* Number of past "lives" with this file */
u16b sf_saves;			/* Number of "saves" during this life */

/*
 * Run-time arguments
 */
bool arg_fiddle;			/* Command arg -- Request fiddle mode */
bool arg_wizard;			/* Command arg -- Request wizard mode */
bool arg_sound;				/* Command arg -- Request special sounds */
bool arg_graphics;			/* Command arg -- Request graphics mode */
bool arg_force_original;	/* Command arg -- Request original keyset */
bool arg_force_roguelike;	/* Command arg -- Request roguelike keyset */

/*
 * Various things
 */
bool character_generated;	/* The character exists */
bool character_dungeon;		/* The character has a dungeon */
bool character_loaded;		/* The character was loaded from a savefile */
bool character_saved;		/* The character was just saved to a savefile */
bool character_existed;		/* A character existed on the same savefile */

s16b character_icky;		/* Depth of the game in special mode */
s16b character_xtra;		/* Depth of the game in startup mode */

u32b seed_alchemy;		/* Hack -- consistent alchemy tables */
u32b seed_flavor;		/* Hack -- consistent object colors */
u32b seed_town;			/* Hack -- consistent town layout */

s16b num_repro;			/* Current reproducer count */
s16b object_level;		/* Current object creation level */
s16b old_object_level;	/* Old object creation level */
s16b monster_level;		/* Current monster creation level */

char summon_kin_type;	/* Hack -- See summon_specific() */

s32b turn;				/* Current game turn */

bool use_sound;			/* The "sound" mode is enabled */
bool use_graphics;		/* The "graphics" mode is enabled */

s16b signal_count;		/* Hack -- Count interrupts */

bool msg_flag;			/* Player has pending message */

bool inkey_base;		/* See the "inkey()" function */
bool inkey_xtra;		/* See the "inkey()" function */
bool inkey_scan;		/* See the "inkey()" function */
bool inkey_flag;		/* See the "inkey()" function */

bool shimmer_monsters;	/* Hack -- optimize multi-hued monsters */
bool shimmer_objects;	/* Hack -- optimize multi-hued objects */

bool repair_mflag_nice;	/* Hack -- repair monster flags (nice) */
bool repair_mflag_show;	/* Hack -- repair monster flags (show) */
bool repair_mflag_mark;	/* Hack -- repair monster flags (mark) */

s16b o_max = 1;			/* Number of allocated objects */
s16b o_cnt = 0;			/* Number of live objects */

s16b mon_max = 1;			/* Number of allocated monsters */
s16b mon_cnt = 0;			/* Number of live monsters */

s16b t_max = 1;			/* Number of allocated traps */
s16b t_cnt = 0;			/* Number of live traps */

/* Hack - temporary monster races for holding uniques */
monster_race monster_temp;
monster_race monster_temp_fake;
monster_lore lore_temp;

/*
 * Hack - Trackees for term windows
 */
s16b term_mon_race_idx;		/* Monster race trackee */
s16b term_mon_unique_idx;	/* Monster unique trackee */
object_type term_object;
bool term_obj_real;

/*
 * TRUE if process_command() is a repeated call.
 */
bool command_repeating = FALSE;

/*
 * Dungeon variables
 */
s16b rating;			/* Level's current rating */
bool good_item_flag;	/* True if "Artifact" on this level */
bool closing_flag;		/* Dungeon is closing */

/*
 * Player info
 */
int player_uid;
int player_euid;
int player_egid;

/*
 * Buffer to hold the current savefile name
 */
char savefile[1024];

/*
 * Number of active macros.
 */
s16b macro__num;

/*
 * Array of macro patterns [MACRO_MAX]
 */
cptr *macro__pat;

/*
 * Array of macro actions [MACRO_MAX]
 */
cptr *macro__act;

/*
 * The array[ANGBAND_TERM_MAX] of window pointers
 */
term *angband_term[ANGBAND_TERM_MAX];

/*
 * The array[8] of window names (modifiable?)
 */
char angband_term_name[ANGBAND_TERM_MAX][16] =
{
	VERSION_NAME,
	"Term-1",
	"Term-2",
	"Term-3",
	"Term-4",
	"Term-5",
  	"Term-6",
	"Term-7"
};

/*
 * Global table of color definitions (mostly zeros)
 */
byte angband_color_table[256][4] =
{
	{0x00, 0x00, 0x00, 0x00},	/* TERM_DARK */
	{0x00, 0xFF, 0xFF, 0xFF},	/* TERM_WHITE */
	{0x00, 0x80, 0x80, 0x80},	/* TERM_SLATE */
	{0x00, 0xFF, 0x80, 0x00},	/* TERM_ORANGE */
	{0x00, 0xC0, 0x00, 0x00},	/* TERM_RED */
	{0x00, 0x00, 0x80, 0x40},	/* TERM_GREEN */
	{0x00, 0x00, 0x40, 0xFF},	/* TERM_BLUE */
	{0x00, 0x80, 0x40, 0x00},	/* TERM_UMBER */
	{0x00, 0x60, 0x60, 0x60},	/* TERM_L_DARK */
	{0x00, 0xC0, 0xC0, 0xC0},	/* TERM_L_WHITE */
	{0x00, 0xFF, 0x00, 0xFF},	/* TERM_VIOLET */
	{0x00, 0xFF, 0xFF, 0x00},	/* TERM_YELLOW */
	{0x00, 0xFF, 0x40, 0x40},	/* TERM_L_RED */
	{0x00, 0x00, 0xFF, 0x00},	/* TERM_L_GREEN */
	{0x00, 0x00, 0xFF, 0xFF},	/* TERM_L_BLUE */
	{0x00, 0xC0, 0x80, 0x40}	/* TERM_L_UMBER */
};


/*
 * Standard sound (and message) names
 */
char angband_sound_name[MSG_MAX][16] =
{
	"generic",
	"fail",
	"succeed",
	"effect",
	"monster",
	"note",
	"bell",
	"timeout",
	"cheat",
	"death",
	"hitpoint_warn",
	"resist",
	"feeling",
	"hit",
	"miss",
	"flee",
	"drop",
	"pickup",
	"curse",
	"kill",
	"level",
	"study",
	"shoot",
	"quaff",
	"zap",
	"throw",
	"fuel",
	"eat",
	"dsm",
	"teleport",
	"tplevel",
	"spell_fail",
	"tpother",
	"hitwall",
	"store",
	"store_angry",
	"store_happy",
	"dig",
	"opendoor",
	"shutdoor",
	"bash",
	"spike",
	"disarm_succeed",
	"disarm_fail",
	"stairs",
	"quest_fail",
	"quest_succeed",
	"trap",
	"critical_hit",
	"find",
	"item_resist",
	"item_damage",
	"item_bonus",
	"item_break",
	"item_recharge",
	"pseudo_id",
	"theft",
	"mon_fail",
	"summon",
	"describe",
	"detect",
	"ffall",
	"room_desc"
};

/*
 * Array[VIEW_MAX] used by "update_view()"
 */
int view_n = 0;
u16b *view_g;

/*
 * Arrays[TEMP_MAX] used for various things
 *
 * Note that temp_g shares memory with temp_x and temp_y.
 */
int temp_n = 0;
u16b *temp_g;
byte *temp_y;
byte *temp_x;

/*
 * Array[DUNGEON_HGT][256] of cave grid info flags (padded)
 *
 * This array is padded to a width of 256 to allow fast access to elements
 * in the array via "grid" values (see the GRID() macros).
 */
byte (*cave_info)[256];

/*
 * Array[DUNGEON_HGT][DUNGEON_WID] of cave grid feature codes
 */
byte (*cave_feat)[MAX_DUNGEON_WID];

/*
 * Array[DUNGEON_HGT][DUNGEON_WID] of cave grid object indexes
 *
 * Note that this array yields the index of the top object in the stack of
 * objects in a given grid, using the "next_o_idx" field in that object to
 * indicate the next object in the stack, and so on, using zero to indicate
 * "nothing".  This array replicates the information contained in the object
 * list, for efficiency, providing extremely fast determination of whether
 * any object is in a grid, and relatively fast determination of which objects
 * are in a grid.
 */
s16b (*cave_o_idx)[MAX_DUNGEON_WID];

/*
 * Array[DUNGEON_HGT][DUNGEON_WID] of cave grid trap indexes
 *
 * Note that this array yields the index of the trap in a grid,.
 * This array replicates the information contained in the trap list, but 
 * provides extremely fast determination of which, if any, trap
 * is in any given grid.
 */
s16b (*cave_t_idx)[MAX_DUNGEON_WID];

/*
 * Array[DUNGEON_HGT][DUNGEON_WID] of cave grid monster indexes
 *
 * Note that this array yields the index of the monster or player in a grid,
 * where negative numbers are used to represent the player, positive numbers
 * are used to represent a monster, and zero is used to indicate "nobody".
 * This array replicates the information contained in the monster list and
 * the player structure, but provides extremely fast determination of which,
 * if any, monster or player is in any given grid.
 */
s16b (*cave_m_idx)[MAX_DUNGEON_WID];

#ifdef MONSTER_FLOW

/*
 * Array[DUNGEON_HGT][DUNGEON_WID] of cave grid flow "cost" values
 */
byte (*cave_cost)[MAX_DUNGEON_WID];

/*
 * Array[DUNGEON_HGT][DUNGEON_WID] of cave grid flow "when" stamps
 */
byte (*cave_when)[MAX_DUNGEON_WID];

#endif	/* MONSTER_FLOW */

/*
 * Array of room information
 *
 * This assumes that room descriptions will never exceed 240 visible characters +
 * 240 invisible characters. Currently we do no bounds checking.
 *
 */
room_info_type room_info[DUN_ROOMS];

/*
 * Array[MAX_ROOMS_ROW][MAX_ROOMS_COL] of room information
 *
 * This indexes into the above room information
 *
 */
byte dun_room[MAX_ROOMS_ROW][MAX_ROOMS_COL];

/*
 * Array[z_info->o_max] of dungeon objects
 */
object_type *o_list;

/*
 * Array[z_info->m_max] of dungeon monsters
 */
monster_type *mon_list;

/*
 * Array[z_info->t_max] of dungeon monsters
 */
trap_type *t_list;

/*
 * Array[z_info->r_max] of monster lore for races
 */
monster_lore *lr_list;

/*
 * Array[z_info->u_max] of monster lore for uniques
 */
monster_lore *lu_list;

/*
 * Array[MAX_STORES] of stores
 */
store_type *store;

/*
 * Array[INVEN_TOTAL] of objects in the player's inventory
 */
object_type *inventory;

/*
 * The arrays of entries in the "kind allocator tables".
 * The first is used for allocation permissions, and the second for final
 * allocation chances.
 */
bool *permit_kind_table;
byte *chance_kind_table;

/*
 * The size of the "alloc_ego_table"
 */
s16b alloc_ego_size;

/*
 * The array[alloc_ego_size] of entries in the "ego allocator table"
 */
alloc_entry *alloc_ego_table;

/*
 * The size of "alloc_race_table" (at most z_info->r_max)
 */
s16b alloc_race_size;

/*
 * The array[alloc_race_size] of entries in the "race allocator table"
 */
alloc_entry *alloc_race_table;

/*
 * The total of all final monster and object generation probabilities
 */
u32b alloc_kind_total;

/*
 * Specify attr/char pairs for visual special effects
 * Be sure to use "index & 0xFF" to avoid illegal access
 */
byte misc_to_attr[256];
char misc_to_char[256];

/*
 * Specify color for inventory item text display (by tval)
 * Be sure to use "index & 0x7F" to avoid illegal access
 */
byte tval_to_attr[128];

/*
 * Current (or recent) macro action
 */
char macro_buffer[1024];

/*
 * Keymaps for each "mode" associated with each keypress.
 */
cptr keymap_act[KEYMAP_MODES][256];

/*** Player information ***/

/*
 * Pointer to the player tables (sex, race, class, magic)
 */
player_sex *sp_ptr;
player_race *rp_ptr;
player_class *cp_ptr;
ptr_player_race_special rsp_ptr[RACE_SPECIAL_LEVELS];

/*
 * The player other record (static)
 */
static player_other player_other_body;

/*
 * Pointer to the player other record
 */
player_other *op_ptr = &player_other_body;

/*
 * The player info record (static)
 */
static player_type player_type_body;

/*
 * Pointer to the player info record
 */
player_type *p_ptr = &player_type_body;


/*
 * Structure (not array) of size limits
 */
maxima *z_info;

/*
 * The vault generation arrays
 */
vault_type *v_info;
char *v_name;
char *v_text;

/*
 * The terrain feature arrays
 */
feature_type *f_info;
char *f_name;

/*
 * The room description information arrays
 */
desc_type *d_info;
char *d_name;
char *d_text;

/*
 * The object kind arrays
 */
object_kind *k_info;
char *k_name;

/*
 * The artifact arrays
 */
artifact_type *a_info;
char *a_name;

/*
 * The ego-item arrays
 */
ego_item_type *e_info;
char *e_name;

/*
 * The prefix arrays
 */
weapon_prefix_type *wpx_info;
char *wpx_name;

/*
 * The prefix arrays
 */
armor_prefix_type *apx_info;
char *apx_name;

/*
 * The trap widget arrays
 */
trap_widget *w_info;
char *w_name;

/*
 * The monster race arrays
 */
monster_race *r_info;
char *r_name;
char *r_text;

/*
 * The monster unique arrays
 */
monster_unique *u_info;
char *u_name;
char *u_text;

/*
 * The monster special arrays
 */
monster_special *s_info;
char *s_name;
char *s_text;

/*
 * The player class arrays
 */
player_class *c_info;
char *c_name;
char *c_text;

/*
 * The player race arrays
 */
player_race *p_info;
char *p_name;

/*
 * The player history arrays
 */
hist_type *h_info;
char *h_text;

/*
 * The shop owner arrays
 */
owner_type *b_info;
char *b_name;

/*
 * The racial price adjustment arrays
 */
byte *g_info;

/*
 * The quest arrays
 */
quest_type *q_info;
char *q_name;

/*
 * Hack -- The special Angband "System Suffix"
 * This variable is used to choose an appropriate "pref-xxx" file
 */
cptr ANGBAND_SYS = "xxx";

/*
 * Hack -- The special Angband "Graphics Suffix"
 * This variable is used to choose an appropriate "graf-xxx" file
 */
cptr ANGBAND_GRAF = "old";

/*
 * Path name: The main "lib" directory
 * This variable is not actually used anywhere in the code
 */
cptr ANGBAND_DIR;

/*
 * High score files (binary)
 * These files may be portable between platforms
 */
cptr ANGBAND_DIR_APEX;

/*
 * Binary image files for the "*_info" arrays (binary)
 * These files are not portable between platforms
 */
cptr ANGBAND_DIR_DATA;

/*
 * Textual template files for the "*_info" arrays (ascii)
 * These files are portable between platforms
 */
cptr ANGBAND_DIR_EDIT;

/*
 * Various extra files (ascii)
 * These files may be portable between platforms
 */
cptr ANGBAND_DIR_FILE;

/*
 * Help files (normal) for the online help (ascii)
 * These files are portable between platforms
 */
cptr ANGBAND_DIR_HELP;

/*
 * Savefiles for current characters (binary)
 * These files are portable between platforms
 */
cptr ANGBAND_DIR_SAVE;

/*
 * Default user "preference" files (ascii)
 * These files are rarely portable between platforms
 */
cptr ANGBAND_DIR_PREF;

/*
 * User defined "preference" files (ascii)
 * These files are rarely portable between platforms
 */
cptr ANGBAND_DIR_USER;

/*
 * Various extra files (binary)
 * These files are rarely portable between platforms
 */
cptr ANGBAND_DIR_XTRA;

/*
 * Total Hack -- allow all items to be listed (even empty ones)
 * This is only used by "do_cmd_inven_e()" and is cleared there.
 */
bool item_tester_full;

/*
 * Here is a "pseudo-hook" used during calls to "get_item()" and
 * "show_inven()" and "show_equip()", and the choice window routines.
 */
byte item_tester_tval;

/*
 * Here is a "hook" used during calls to "get_item()" and
 * "show_inven()" and "show_equip()", and the choice window routines.
 */
bool (*item_tester_hook)(const object_type*);

/*
 * Current "comp" function for ang_sort()
 */
bool (*ang_sort_comp)(const void *u, const void *v, int a, int b);

/*
 * Current "swap" function for ang_sort()
 */
void (*ang_sort_swap)(void *u, void *v, int a, int b);

/*
 * Hack -- function hook to restrict "get_mon_num_prep()" function
 */
bool (*get_mon_num_hook)(int r_idx);

/*
 * Hack -- function hook to restrict "get_obj_num_prep()" function
 */
bool (*get_obj_num_hook)(int k_idx);
bool(*old_get_obj_num_hook)(int k_idx);

/*
 * Hack - the destination file for text_out_to_file.
 */
FILE *text_out_file = NULL;

/*
 * Hack -- function hook to output (colored) text to the
 * screen or to a file.
 */
void (*text_out_hook)(byte a, cptr str);

/* Hack -- Where to wrap the text when using text_out().  Use the default 
 * value (for example the screen width) when 'text_out_wrap' is 0. 
 * 
 */ 
int text_out_wrap = 0; 
    
/* 
 * Hack -- Indentation for the text when using text_out(). 
 * 
 */ 
int text_out_indent = 0; 
  
/*
 * The "highscore" file descriptor, if available.
 */
int highscore_fd = -1;

/*
 * Use transparent tiles
 */
bool use_transparency = FALSE;

/*
 * Game can be saved
 */
bool can_save = TRUE;

/*
 * The alchemy information
 */
alchemy_info potion_alch[SV_POTION_MAX];
